@@ -28,7 +28,9 @@ char acct_sccsid[] =
 #include	"conf.h"
 
 #include	"radiusd.h"
+#if 0
 #include	"radutmp.h"
+#endif
 
 /*
  *	FIXME: this should be configurable.
