@@ -73,7 +73,7 @@ REALM		*realms;
 /*
  *	Returns a copy of a pair list.
  */
-static VALUE_PAIR *paircopy(VALUE_PAIR *from)
+VALUE_PAIR *paircopy(VALUE_PAIR *from)
 {
 	VALUE_PAIR *vp = NULL;
 	VALUE_PAIR *last = NULL;
@@ -323,10 +323,8 @@ static int groupcmp(VALUE_PAIR *check, char *username)
 	if (cache_passwd && (retval = H_groupcmp(check, username)) != -2)
 		return retval;
 
-#ifndef USEMYSQL
 	if ((pwd = rad_getpwnam(username)) == NULL)
 		return -1;
-#endif
 
 	if ((grp = getgrnam(check->strvalue)) == NULL)
 		return -1;
@@ -482,11 +480,6 @@ int paircmp(VALUE_PAIR *request, VALUE_PAIR *check) {
 						return -1;
 				}
 				else
-				if (check_item->attribute == PW_NAS_PORT_ID) {
-					if (portcmp(check_item, auth_item) != 0)
-						return -1;
-				}
-			 	else
 				if (check_item->attribute == PW_GROUP_NAME ||
 				    check_item->attribute == PW_GROUP) {
 					compare = groupcmp(check_item, username);
@@ -503,6 +496,11 @@ int paircmp(VALUE_PAIR *request, VALUE_PAIR *check) {
 				break;
 
 			case PW_TYPE_INTEGER:
+				if (check_item->attribute == PW_NAS_PORT_ID) {
+					compare = portcmp(check_item,auth_item);
+					break;
+				}
+				/*FALLTHRU*/
 			case PW_TYPE_IPADDR:
 				compare = auth_item->lvalue - check_item->lvalue;
 				break;
@@ -604,11 +602,6 @@ static int hunt_paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 		switch(check_item->type) {
 
 			case PW_TYPE_STRING:
-				if (check_item->attribute == PW_NAS_PORT_ID) {
-					if (portcmp(check_item, auth_item) == 0)
-						result = 0;
-				}
-			 	else
 				if (check_item->attribute == PW_GROUP_NAME ||
 				    check_item->attribute == PW_GROUP) {
 					if (groupcmp(check_item, auth_item->strvalue)==0)
@@ -628,6 +621,12 @@ static int hunt_paircmp(VALUE_PAIR *request, VALUE_PAIR *check)
 				break;
 
 			case PW_TYPE_INTEGER:
+				if (check_item->attribute == PW_NAS_PORT_ID) {
+					if (portcmp(check_item, auth_item) == 0)
+						result = 0;
+					break;
+				}
+				/*FALLTHRU*/
 			case PW_TYPE_IPADDR:
 				if(check_item->lvalue == auth_item->lvalue) {
 					result = 0;
@@ -903,88 +902,6 @@ parse_again:
 }
 
 
-/*
- *	Find the named user in the DBM user database.
- *	Returns: -1 not found
- *	          0 found but doesn't match.
- *	          1 found and matches.
- */
-#ifdef USE_DBM
-static int dbm_find(char *name, VALUE_PAIR *request_pairs,
-		VALUE_PAIR **check_pairs, VALUE_PAIR **reply_pairs)
-{
-	datum		named;
-	datum		contentd;
-	char		*ptr;
-	VALUE_PAIR	*check_tmp;
-	VALUE_PAIR	*reply_tmp;
-	int		ret = 0;
-
-	named.dptr = name;
-	named.dsize = strlen(name);
-#ifdef DBM
-	contentd = fetch(named);
-#endif
-#ifdef NDBM
-	contentd = dbm_fetch(dbmfile, named);
-#endif
-	if(contentd.dptr == NULL)
-		return -1;
-
-	check_tmp = NULL;
-	reply_tmp = NULL;
-
-	/*
-	 *	Parse the check values
-	 */
-	ptr = contentd.dptr;
-	contentd.dptr[contentd.dsize] = '\0';
-
-	if (*ptr != '\n' && userparse(ptr, &check_tmp) != 0) {
-		log(L_ERR|L_CONS, "Parse error (check) for user %s", name);
-		pairfree(check_tmp);
-		return -1;
-	}
-	while(*ptr != '\n' && *ptr != '\0') {
-		ptr++;
-	}
-	if(*ptr != '\n') {
-		log(L_ERR|L_CONS, "Parse error (no reply pairs) for user %s",
-			name);
-		pairfree(check_tmp);
-		return -1;
-	}
-	ptr++;
-
-	/*
-	 *	Parse the reply values
-	 */
-	if (userparse(ptr, &reply_tmp) != 0) {
-		log(L_ERR|L_CONS, "Parse error (reply) for user %s", name);
-		pairfree(check_tmp);
-		pairfree(reply_tmp);
-		return -1;
-	}
-
-	/*
-	 *	See if the check_pairs match.
-	 */
-	auth_type_fixup(check_tmp);
-	if (paircmp(request_pairs, check_tmp) == 0) {
-		ret = 1;
-		pairmove(reply_pairs, &reply_tmp);
-		pairmove2(reply_pairs, &reply_tmp, PW_FALL_THROUGH);
-		pairmove(check_pairs, &check_tmp);
-	}
-	pairfree(reply_tmp);
-	pairfree(check_tmp);
-
-	return ret;
-}
-#endif /* DBM */
-
-
-
 #ifdef ASCEND_PORT_HACK
 /*
  *	dgreer --
@@ -1015,13 +932,14 @@ static int ascend_port_number(int nas_port)
  *	for this user from the database. The main code only
  *	needs to check the password, the rest is done here.
  */
-int user_find(char *name, VALUE_PAIR *request_pairs,
-		VALUE_PAIR **check_pairs, VALUE_PAIR **reply_pairs,  SQLSOCK *socket)
-{
+int user_find(char *name, AUTH_REQ *authreq, SQLSOCK *socket) {
 	int		nas_port = 0;
 	VALUE_PAIR	*check_tmp = NULL;
 	VALUE_PAIR	*reply_tmp = NULL;
+	VALUE_PAIR	**check_pairs;
+	VALUE_PAIR	**reply_pairs;
 	VALUE_PAIR	*tmp;
+	VALUE_PAIR	*request_pairs;
 	int		found = 0;
 #ifdef NT_DOMAIN_HACK
 	char		*ptr;
@@ -1029,6 +947,10 @@ int user_find(char *name, VALUE_PAIR *request_pairs,
 #endif
 
 
+	check_pairs = &(authreq->check_pairs);
+	reply_pairs = &(authreq->reply_pairs);
+	request_pairs = authreq->request;
+
 	/* 
 	 *	Check for valid input, zero length names not permitted 
 	 */
@@ -1256,26 +1178,23 @@ int userparse(char *buffer, VALUE_PAIR **first_pair)
 			pair->attribute = attr->value;
 			pair->type = attr->type;
 			pair->operator = operator;
+			pair->source = 0;
+
+			/* _Always_ copy value as string. */
+			strNcpy(pair->strvalue, valstr, sizeof(pair->strvalue));
+
 			switch(pair->type) {
 
 			case PW_TYPE_STRING:
-do_copy:
-                                strncpy(pair->strvalue, valstr,
-                                        sizeof(pair->strvalue));
-                                pair->strvalue[sizeof(pair->strvalue) - 1] = 0;
+				/* Pair->strvalue has been set already */
 				pair->length = strlen(pair->strvalue);
 				break;
 
 			case PW_TYPE_INTEGER:
 				/*
-				 *	For PW_NAS_PORT_ID, allow a
-				 *	port range instead of just a port.
+				 *	If it starts with a digit, it must
+				 *	be a number (or a range).
 				 */
-				if (attr->value == PW_NAS_PORT_ID) {
-					for(s = valstr; *s; s++)
-						if (!isdigit(*s)) break;
-					if (*s) goto do_copy;
-				}
 				if (isdigit(*valstr)) {
 					pair->lvalue = atoi(valstr);
 					pair->length = 4;
@@ -1329,6 +1248,7 @@ do_copy:
 				pair2->type = PW_TYPE_INTEGER;
 				pair2->lvalue = x;
 				pair2->length = 4;
+				pair2->source = 0;
 				pairadd(first_pair, pair2);
 				break;
 
@@ -1386,8 +1306,9 @@ static int matches(char *name, PAIR_LIST *pl, char *matchpart, int matchlen)
 	if ((tmp = pairfind(pl->check, PW_PREFIX)) != NULL ||
 	    (tmp = pairfind(pl->check, PW_SUFFIX)) != NULL) {
 
-		if (strcmp(pl->name, name) == 0)
+		if (strncmp(pl->name, "DEFAULT", 7) == 0 || strcmp(pl->name, name) == 0)
 			return !presufcmp(tmp, name, matchpart, matchlen);
+
 	}
 
 	/*
@@ -1435,7 +1356,7 @@ static int matches(char *name, PAIR_LIST *pl, char *matchpart, int matchlen)
  *	Add hints to the info sent by the terminal server
  *	based on the pattern of the username.
  */
-int hints_setup(VALUE_PAIR *request_pairs)
+int hints_setup(AUTH_REQ *authreq)
 {
 	char		newname[AUTH_STRING_LEN];
 	char		*name;
@@ -1444,6 +1365,7 @@ int hints_setup(VALUE_PAIR *request_pairs)
 	VALUE_PAIR	*tmp;
 	PAIR_LIST	*i;
 	int		do_strip;
+	VALUE_PAIR	*request_pairs;
 #if defined(NT_DOMAIN_HACK) || defined(SPECIALIX_JETSTREAM_HACK)
 	char		*ptr;
 #endif
@@ -1451,6 +1373,8 @@ int hints_setup(VALUE_PAIR *request_pairs)
 	if (hints == NULL)
 		return 0;
 
+	request_pairs = authreq->request;
+
 	/* 
 	 *	Check for valid input, zero length names not permitted 
 	 */
@@ -1514,8 +1438,7 @@ int hints_setup(VALUE_PAIR *request_pairs)
 
 	for (i = hints; i; i = i->next) {
 		if (matches(name, i, newname, sizeof(newname))) {
-			DEBUG2("  hints: Matched %s at %d",
-			       i->name, i->lineno);
+			DEBUG2("  hints: Matched %s at %d", i->name, i->lineno);
 			break;
 		}
 	}
@@ -1833,17 +1756,16 @@ int read_config_files(SQLSOCK *socket) {
 	pairlist_free(&huntgroups);
 	pairlist_free(&hints);
 
+	DEBUG2("Reading huntgroups file %s/%s", radius_dir, RADIUS_HUNTGROUPS);
 	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_HUNTGROUPS);
 	huntgroups = file_read(buffer, 0);
-	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_HINTS);
-	hints      = file_read(buffer, 0);
+
+	hints = sql_read_hints(socket);
 
 	if (sql_read_realms(socket) < 0)
 		return -1;
 	if (sql_read_naslist(socket) < 0)
 		return -1;
-	if (sql_read_clients(socket) < 0)
-		return -1;
 
 	if (cache_passwd) {
 		log(L_INFO, "HASH:  Reinitializing hash structures "
