@@ -50,7 +50,11 @@ int sql_init(int reload) {
 	strcpy(sql->config->sql_server,"localhost");
 	strcpy(sql->config->sql_login,"");
 	strcpy(sql->config->sql_password,"");
+	strcpy(sql->config->sql_acct_server,"localhost");
+	strcpy(sql->config->sql_acct_login,"");
+	strcpy(sql->config->sql_acct_password,"");
 	strcpy(sql->config->sql_db,"radius");
+	strcpy(sql->config->sql_acct_db,"radius");
 	strcpy(sql->config->sql_authcheck_table,"radcheck");
 	strcpy(sql->config->sql_authreply_table,"radreply");
 	strcpy(sql->config->sql_groupcheck_table,"radgroupcheck");
@@ -66,6 +70,10 @@ int sql_init(int reload) {
 	strcpy(sql->config->sql_nas_table,"nas");
 	strcpy(sql->config->sql_realm_table, "realms");
 	strcpy(sql->config->sql_dict_table,"dictionary");
+	if (reload)
+		sql->config->max_sql_oldsocks = sql->config->max_sql_socks;
+	else
+		sql->config->max_sql_oldsocks = MAX_SQL_SOCKS;
 	sql->config->max_sql_socks = MAX_SQL_SOCKS;
 
         sprintf(sqlfile, "%s/%s", radius_dir, MYSQLCONFIG);
@@ -89,6 +97,7 @@ int sql_init(int reload) {
                                log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strcpy(sql->config->sql_server,namestr);
+                         strcpy(sql->config->sql_acct_server,namestr);
                        }
                }
                 if(strncasecmp(buffer, "login", 5) == 0) {
@@ -97,6 +106,7 @@ int sql_init(int reload) {
                                log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strcpy(sql->config->sql_login,namestr);
+                         strcpy(sql->config->sql_acct_login,namestr);
                        }
                }
                 if(strncasecmp(buffer, "password", 8) == 0) {
@@ -105,6 +115,31 @@ int sql_init(int reload) {
                                log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
                        } else {
                          strcpy(sql->config->sql_password,namestr);
+                         strcpy(sql->config->sql_acct_password,namestr);
+                       }
+               }
+                if(strncasecmp(buffer, "acct_server", 11) == 0) {
+                        /* Read the SERVER line */
+                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
+                       } else {
+                         strcpy(sql->config->sql_acct_server,namestr);
+                       }
+               }
+                if(strncasecmp(buffer, "acct_login", 10) == 0) {
+                        /* Read the LOGIN line */
+                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
+                       } else {
+                         strcpy(sql->config->sql_acct_login,namestr);
+                       }
+               }
+                if(strncasecmp(buffer, "acct_password", 13) == 0) {
+                        /* Read the PASSWORD line */
+                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
+                       } else {
+                         strcpy(sql->config->sql_acct_password,namestr);
                        }
                }
                 if(strncasecmp(buffer, "radius_db", 9) == 0) {
@@ -114,6 +149,14 @@ int sql_init(int reload) {
                        } else {
                          strcpy(sql->config->sql_db,namestr);
                        }
+               }
+                if(strncasecmp(buffer, "radius_acct_db", 14) == 0) {
+                        /* Read the RADIUS_DB line */
+                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
+                       } else {
+                         strcpy(sql->config->sql_acct_db,namestr);
+                       }
                }
                 if(strncasecmp(buffer, "authcheck_table", 15) == 0) {
                         /* Read the AUTHCHECK_TABLE line */
@@ -239,6 +282,14 @@ int sql_init(int reload) {
                         }
                        }
                }
+               if(strncasecmp(buffer, "max_sql_sockets", 15) == 0) {
+                        /* Read the MAX_SQL_SOCKETS line */
+                        if(sscanf(buffer, "%s%s", dummystr, namestr) != 2) {
+                               log(L_ERR,"invalid attribute on line %d of configuration file %s", line_no,sqlfile);
+                       } else {
+                           sql->config->max_sql_socks = atoi(namestr);
+                       }
+               }
 
        }
        fclose(sqlfd);
@@ -264,13 +315,25 @@ int sql_init_socket(int reload) {
 
 	/* Clear up old connections if reload */
 	if (reload)
-	        for (i = 0; i < sql->config->max_sql_socks; i++)
+	        for (i = 0; i < sql->config->max_sql_oldsocks; i++) {
 			if (!sql_close_socket(sql->socks[i]))
 				log(L_CONS|L_ERR, "Could not release socket %d", i);
+		}
+		free(sql->socks);
 	
 
         /* Initalize our connection pool */
+	sql->socks = malloc(sizeof(SQLSOCK *) * sql->config->max_sql_socks);
+	if(!sql->socks) {
+		log(L_ERR,"Unable to allocate memory for sockets structure");
+		return -1;
+	}
+	bzero(sql->socks,sizeof(SQLSOCK *) * sql->config->max_sql_socks);
         for (i = 0; i < sql->config->max_sql_socks; i++) {
+		sql->socks[i] = malloc(sizeof(SQLSOCK));
+		if(!sql->socks[i])
+			log(L_ERR,"Unable to allocate memory for sockets structure");
+		bzero(sql->socks[i],sizeof(SQLSOCK));
 	 	if ((sql->socks[i] = sql_create_socket()) == NULL) {
 	   		log(L_CONS|L_ERR, "SQL: Failed to connect socket %d", i);
 		} else {
@@ -279,8 +342,8 @@ int sql_init_socket(int reload) {
 	   		DEBUG2("SQL: Connected socket %d", i);
 		}
 	}
-           
-       return 1;
+
+	return 1;
 }
 
 
@@ -321,6 +384,7 @@ SQLSOCK *sql_get_socket(void) {
 			return sql->socks[i];
 		}
 		i++;
+		sleep(1);
 	}
 }
 
