@@ -186,8 +186,8 @@ int proxy_addrequest(AUTH_REQ *authreq, int *proxy_id)
 static void passwd_recode(char *secret_key, char *vector,
 		char *pw_digest, char *pass, int passlen)
 {
-	char	passtmp[AUTH_PASS_LEN];
-	char	passwd[AUTH_PASS_LEN];
+	char	passtmp[AUTH_PASS_LEN + 1];
+	char	passwd[AUTH_PASS_LEN + 1];
 	char	md5buf[256];
 	int	i;
 	int	len;
