@@ -72,6 +72,8 @@ int			log_auth = 0;
 int			log_auth_pass  = 0;
 int			auth_port;
 int			acct_port;
+int			acct_pid;
+int			radius_pid;
 
 /*
  *	Make sure recv_buffer is aligned properly.
@@ -85,8 +87,6 @@ static int		sockfd;
 static int		acctfd;
 static int		spawn_flag;
 static int		fork_auth;
-static int		acct_pid;
-static int		radius_pid;
 static int		need_reload = 0;
 static AUTH_REQ		*first_request;
 
@@ -420,10 +420,12 @@ int main(int argc, char **argv)
 			log(L_INFO, "Ready to process requests.");
 		}
 		else {
+			acct_pid = getpid();
 			acctsock = sql_create_socket();
 			acctsock->id = sql->config->max_sql_socks;
 			acctsock->in_use = getpid();
 			DEBUG2("SQL: Connected socket %d", sql->config->max_sql_socks);
+			acct_pid = 0;
 			close(sockfd);
 			sockfd = -1;
 		}
