@@ -13,7 +13,11 @@ typedef struct sql_config {
         char sql_server[40];
         char sql_login[20];
         char sql_password[20];
+        char sql_acct_server[40];
+        char sql_acct_login[20];
+        char sql_acct_password[20];
         char sql_db[20];
+        char sql_acct_db[20];
         char sql_acct_table[MAX_TABLE_LEN];
         char sql_authcheck_table[MAX_TABLE_LEN];
         char sql_authreply_table[MAX_TABLE_LEN];
@@ -29,6 +33,7 @@ typedef struct sql_config {
         int  sqltrace;
 	int  deletestalesessions;
 	int  max_sql_socks;
+	int  max_sql_oldsocks;
 } SQL_CONFIG;
  
 typedef struct sql_socket {
@@ -42,7 +47,7 @@ typedef struct sql_socket {
  
 typedef struct sql {
         SQL_CONFIG *config;
-        SQLSOCK *socks[MAX_SQL_SOCKS];
+        SQLSOCK **socks;
 } SQL;
 
 
